﻿using DbContextLib;
using Entities;
using Microsoft.EntityFrameworkCore;
using Model.OrderCriteria;
using Shared;

namespace DbDataManager.Service;

public class LessonDataService : ILessonService<LessonEntity>
{
    private UserDbContext DbContext { get; set; }

    public LessonDataService(UserDbContext context)
    {
        DbContext = context;
        context.Database.EnsureCreated();
    }

    public LessonEntity GetLessonById(int id)
    {
        var lessonEntity = DbContext.Lessons.FirstOrDefault(u => u.Id == id);
        if (lessonEntity == null)
        {
            throw new ArgumentException("Impossible de trouver la leçon", nameof(id));
        }

        return lessonEntity;
    }

    public LessonEntity GetLessonByTitle(string title)
    {
        var lessonEntity = DbContext.Lessons.FirstOrDefault(u => u.Title == title);
        if (lessonEntity == null)
        {
            throw new ArgumentException("Impossible de trouver la leçon", nameof(title));
        }

        return lessonEntity;
    }

    public IEnumerable<LessonEntity> GetLessons(int page, int number, LessonOrderCriteria orderCriteria)
    {
        return DbContext.Lessons.Skip((page - 1) * number).Take(number).ToList()
            .Select(u => u);
    }

    public bool DeleteLesson(int id)
    {
        var lessonEntity = DbContext.Lessons.FirstOrDefault(u => u.Id == id);
        if (lessonEntity == null)
        {
            return false;
        }

        DbContext.Lessons.Remove(lessonEntity);
        DbContext.SaveChangesAsync();
        return true;
    }

    public LessonEntity UpdateLesson(int id, LessonEntity lesson)
    {
        var updatingLesson = DbContext.Lessons.FirstOrDefault(u => u.Id == id);
        if (updatingLesson == null)
        {
            throw new ArgumentException("Impossible de trouver la leçon", nameof(id));
        }

        updatingLesson.Title = lesson.Title;
        updatingLesson.LastPublisher = lesson.LastPublisher;
        updatingLesson.LastEdit = lesson.LastEdit;
        // Permet d'indiquer en Db que l'entité a été modifiée.
        DbContext.Entry(updatingLesson).State = EntityState.Modified;
        DbContext.SaveChangesAsync();
        return updatingLesson;
    }

    public LessonEntity CreateLesson(string title, string lastPublisher, DateOnly? lastEdit)
    {
        var newLessonEntity = new LessonEntity()
        {
            Title = title,
            LastPublisher = lastPublisher,
            LastEdit = lastEdit,
        };
        DbContext.Lessons.Add(newLessonEntity);
        DbContext.SaveChangesAsync();
        return newLessonEntity;
    }
}