﻿using DbContextLib;
using Entities;
using Microsoft.EntityFrameworkCore;
using Model.OrderCriteria;
using Shared;

namespace DbDataManager.Service;

public class ParagraphDataService : IParagraphService<ParagraphEntity>
{
    private UserDbContext DbContext { get; set; }

    public ParagraphDataService(UserDbContext context)
    {
        DbContext = context;
        context.Database.EnsureCreated();
    }

    public ParagraphEntity GetParagraphById(int id)
    {
        var paragraphEntity = DbContext.Paragraphs.FirstOrDefault(u => u.Id == id);
        if (paragraphEntity == null)
        {
            throw new ArgumentException("Impossible de trouver le paragraphe", nameof(id));
        }

        return paragraphEntity;
    }

    public ParagraphEntity GetParagraphByTitle(string title)
    {
        var paragraphEntity = DbContext.Paragraphs.FirstOrDefault(u => u.Title == title);
        if (paragraphEntity == null)
        {
            throw new ArgumentException("Impossible de trouver le paragraphe", nameof(title));
        }

        return paragraphEntity;
    }

    public IEnumerable<ParagraphEntity> GetParagraphs(int page, int number, ParagraphOrderCriteria orderCriteria)
    {
        return DbContext.Paragraphs.Skip((page - 1) * number).Take(number).ToList()
            .Select(u => u);
    }

    public bool DeleteParagraph(int id)
    {
        var paragraphEntity = DbContext.Paragraphs.FirstOrDefault(u => u.Id == id);
        if (paragraphEntity == null)
        {
            return false;
        }

        DbContext.Paragraphs.Remove(paragraphEntity);
        DbContext.SaveChangesAsync();
        return true;
    }

    public ParagraphEntity UpdateParagraph(int id, ParagraphEntity paragraph)
    {
        var updatingParagraph = DbContext.Paragraphs.FirstOrDefault(u => u.Id == id);
        if (updatingParagraph == null)
        {
            throw new ArgumentException("Impossible de trouver le paragraphe", nameof(id));
        }

        updatingParagraph.Title = paragraph.Title;
        updatingParagraph.Content = paragraph.Content;
        updatingParagraph.Info = paragraph.Info;
        updatingParagraph.Query = paragraph.Query;
        updatingParagraph.Comment = paragraph.Comment;
        updatingParagraph.LessonId = paragraph.LessonId;
        // Permet d'indiquer en Db que l'entité a été modifiée.
        DbContext.Entry(updatingParagraph).State = EntityState.Modified;
        DbContext.SaveChangesAsync();
        return updatingParagraph;
    }

    public ParagraphEntity CreateParagraph(string title, string content, string info, string query, string comment, int lessonId)
    {
        var newParagraphEntity = new ParagraphEntity()
        {
            Title = title,
            Content = content,
            Info = info,
            Query = query,
            Comment = comment,
            LessonId = lessonId
        };
        DbContext.Paragraphs.Add(newParagraphEntity);
        DbContext.SaveChangesAsync();
        return newParagraphEntity;
    }
}