﻿using DbContextLib;
using Entities;
using Microsoft.EntityFrameworkCore;
using Model.OrderCriteria;
using Shared;

namespace DbDataManager.Service;

public class SuccessDataService : ISuccessService<SuccessEntity>
{
    private UserDbContext DbContext { get; set; }

    public SuccessDataService(UserDbContext context)
    {
        DbContext = context;
        context.Database.EnsureCreated();
    }

    public SuccessEntity GetSuccessByUserId(int id)
    {
        var successEntity = DbContext.Successes.FirstOrDefault(u => u.UserId == id);
        if (successEntity == null)
        {
            throw new ArgumentException("Impossible de trouver le succès avec l'user id", nameof(id));
        }

        return successEntity;
    }

    public SuccessEntity GetSuccessByInquiryId(int id)
    {
        var successEntity = DbContext.Successes.FirstOrDefault(u => u.InquiryId == id);
        if (successEntity == null)
        {
            throw new ArgumentException("Impossible de trouver le succès avec l'inquiry id", nameof(id));
        }

        return successEntity;
    }

    public IEnumerable<SuccessEntity> GetSuccesses(int page, int number, SuccessOrderCriteria orderCriteria)
    {
        return DbContext.Successes.Skip((page - 1) * number).Take(number).ToList()
            .Select(u => u);
    }

    public bool DeleteSuccess(int id)
    {
        var userEntity = DbContext.Successes.FirstOrDefault(u => u.UserId == id);
        if (userEntity == null)
        {
            return false;
        }

        DbContext.Successes.Remove(userEntity);
        DbContext.SaveChangesAsync();
        return true;
    }

    public SuccessEntity UpdateSuccess(int id, SuccessEntity success)
    {
        var updatingSuccess = DbContext.Successes.FirstOrDefault(u => u.UserId == id);
        if (updatingSuccess == null)
        {
            throw new ArgumentException("Impossible de trouver le succès avec l'user id", nameof(id));
        }

        updatingSuccess.UserId = success.UserId;
        updatingSuccess.InquiryId = success.InquiryId;
        updatingSuccess.IsFinished = success.IsFinished;
        // Permet d'indiquer en Db que l'entité a été modifiée.
        DbContext.Entry(updatingSuccess).State = EntityState.Modified;
        DbContext.SaveChangesAsync();
        return updatingSuccess;
    }

    public SuccessEntity CreateSuccess(int userId, int inquiryId, bool isFinished)
    {
        var newSuccessEntity = new SuccessEntity()
        {
            UserId = userId,
            InquiryId = inquiryId,
            IsFinished = isFinished,
        };
        DbContext.Successes.Add(newSuccessEntity);
        DbContext.SaveChangesAsync();
        return newSuccessEntity;
    }
}