﻿using System.Runtime.CompilerServices;
using System.Security.Cryptography;
using DbContextLib;
using Entities;
using Microsoft.AspNetCore.Cryptography.KeyDerivation;
using Microsoft.EntityFrameworkCore;

namespace StubbedContextLib;

public class StubbedContext : UserDbContext
{
    public StubbedContext(DbContextOptions<UserDbContext> options) : base(options)
    {
    }
    public StubbedContext() : base() { }

    protected override void OnModelCreating(ModelBuilder modelBuilder)
    {
        base.OnModelCreating(modelBuilder);
        modelBuilder.Entity<UserEntity>().HasData(
            new UserEntity(1, "johnny", Convert.ToBase64String(KeyDerivation.Pbkdf2(
                password: "motdepasse",
                salt: RandomNumberGenerator.GetBytes(128 / 8),
                prf: KeyDerivationPrf.HMACSHA256,
                iterationCount: 100000,
                numBytesRequested: 256 / 8)), "Johnny.RATTON@etu.uca.fr", true),
            new UserEntity(2, "maxime", Convert.ToBase64String(KeyDerivation.Pbkdf2(
                password: "motdepasse",
                salt: RandomNumberGenerator.GetBytes(128 / 8),
                prf: KeyDerivationPrf.HMACSHA256,
                iterationCount: 100000,
                numBytesRequested: 256 / 8)), "Maxime.SAPOUNTZIS@etu.uca.fr", true),
            new UserEntity(3, "clement", Convert.ToBase64String(KeyDerivation.Pbkdf2(
                password: "motdepasse",
                salt: RandomNumberGenerator.GetBytes(128 / 8),
                prf: KeyDerivationPrf.HMACSHA256,
                iterationCount: 100000,
                numBytesRequested: 256 / 8)), "Clement.CHIEU@etu.uca.fr", true),
            new UserEntity(4, "erwan", Convert.ToBase64String(KeyDerivation.Pbkdf2(
                password: "motdepasse",
                salt: RandomNumberGenerator.GetBytes(128 / 8),
                prf: KeyDerivationPrf.HMACSHA256,
                iterationCount: 100000,
                numBytesRequested: 256 / 8)), "Erwan.MENAGER@etu.uca.fr", true),
            new UserEntity(5, "victor", Convert.ToBase64String(KeyDerivation.Pbkdf2(
                password: "motdepasse",
                salt: RandomNumberGenerator.GetBytes(128 / 8),
                prf: KeyDerivationPrf.HMACSHA256,
                iterationCount: 100000,
                numBytesRequested: 256 / 8)), "Victor.GABORIT@etu.uca.fr", true));

        modelBuilder.Entity<InquiryEntity>().HasData(
            new InquiryEntity(1, "L'enquête de la carotte", "La description de l'inquiry1", true, 1, 1),
            new InquiryEntity(2, "L'enquête sur les orang outan", "The new description", false, 2, 2),
            new InquiryEntity(3, "L'enquête sur les parapluies", "Il pleuvait", false, 3, 3));

        modelBuilder.Entity<InquiryTableEntity>().HasData(
            new InquiryTableEntity(1, "Inquiry1",
                "Server=localhost;Database=Inquiry1;Trusted_Connection=True;MultipleActiveResultSets=true"),
            new InquiryTableEntity(2, "Inquiry2",
                "Server=localhost;Database=Inquiry2;Trusted_Connection=True;MultipleActiveResultSets=true"),
            new InquiryTableEntity(3, "Inquiry3",
                "Server=localhost;Database=Inquiry3;Trusted_Connection=True;MultipleActiveResultSets=true"));

        modelBuilder.Entity<SolutionEntity>().HasData(
            new SolutionEntity(1, "Maxime", "Sapountzis", "La cuisine", "Le couteau", "Parce que c'est Maxime"),
            new SolutionEntity(2, "Johnny", "Ratton", "La cuisine", "Le couteau", "Parce que il est ra****"),
            new SolutionEntity(3, "Erwan", "Menager", "La salle de bain", "L'arachide", "Parce que c'est Erwan"));
    }
}