﻿// See https://aka.ms/new-console-template for more information

using API.Controllers;
using DbContextLib;
using Dto;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Data.Sqlite;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Logging;
using Model.OrderCriteria;
using Shared;

var connection = new SqliteConnection("DataSource=:memory:");
connection.Open();
var options = new DbContextOptionsBuilder<UserDbContext>()
                    .UseSqlite(connection)
                    .Options;
using ILoggerFactory factory = new LoggerFactory();
ILogger<UsersController> userLogger = factory.CreateLogger<UsersController>();
ILogger<InquiriesController> inquiryLogger = factory.CreateLogger<InquiriesController>();
ILogger<ParagraphsController> paragraphLogger = factory.CreateLogger<ParagraphsController>();
ILogger<LessonsController> lessonLogger = factory.CreateLogger<LessonsController>();
ILogger<SuccessesController> successLogger = factory.CreateLogger<SuccessesController>();


using (var context = new UserDbContext(options))
{
    var userController = new UsersController(userLogger, new UserDataService(context));
    var inquiryController = new InquiriesController(new InquiryDataService(context), inquiryLogger);
    var paragraphController = new ParagraphsController(new ParagraphDataService(context), paragraphLogger);
    var lessonController = new LessonsController(new LessonDataService(context), lessonLogger);
    var successController = new SuccessesController(new SuccessDataService(context), successLogger);

    void PrintUsers()
    {
        Console.WriteLine();
        var users = userController.GetUsers(1, 10, UserOrderCriteria.None) as OkObjectResult;
        foreach (var item in users.Value as IEnumerable<UserDTO>)
        {
            Console.WriteLine(item);
        }
    }

    void PrintInquiries()
    {
        Console.WriteLine();
        var inquiries = inquiryController.GetInquiries(1, 10, InquiryOrderCriteria.None) as OkObjectResult;
        foreach (var item in inquiries.Value as IEnumerable<InquiryDTO>)
        {
            Console.WriteLine(item);
        }
    }

    void PrintParagraphs()
    {
        Console.WriteLine();
        var paragraphs = paragraphController.GetParagraphs(1, 10, ParagraphOrderCriteria.None) as OkObjectResult;
        foreach (var item in paragraphs.Value as IEnumerable<ParagraphDTO>)
        {
            Console.WriteLine(item);
        }
    }

    void PrintLessons()
    {
        Console.WriteLine();
        var lessons = lessonController.GetLessons(1, 10, LessonOrderCriteria.None) as OkObjectResult;
        foreach (var item in lessons.Value as IEnumerable<LessonDTO>)
        {
            Console.WriteLine(item);
        }
    }

    void PrintSuccesses()
    {
        Console.WriteLine();
        var successes = successController.GetSuccesses(1, 10, SuccessOrderCriteria.None) as OkObjectResult;
        foreach (var item in successes.Value as IEnumerable<SuccessDTO>)
        {
            Console.WriteLine(item);
        }
    }

    void SearchUserByUsername()
    {
        Console.WriteLine("\nVeuillez saisir le pseudonyme de l'utilisateur recherché : ");
        var username = Console.ReadLine();
        var user = userController.GetUserByUsername(username) as OkObjectResult;
        if (user == null)
        {
            Console.WriteLine("Erreur, la requête n'a rien donné.");
            return;
        }
        Console.WriteLine(user.Value as UserDTO);

    }

    void SearchInquiryByTitle()
    {
        Console.WriteLine("\nVeuillez saisir le titre de l'enquête recherchée : ");
        var title = Console.ReadLine();
        var inquiry = inquiryController.GetInquiryByTitle(title) as OkObjectResult;
        if (inquiry == null)
        {
            Console.WriteLine("Erreur, la requête n'a rien donné.");
            return;
        }
        Console.WriteLine(inquiry.Value as InquiryDTO);
    }

    void SearchParagraphByTitle()
    {
        Console.WriteLine("\nVeuillez saisir le titre du paragraphe recherché : ");
        var title = Console.ReadLine();
        var paragraph = paragraphController.GetParagraphByTitle(title) as OkObjectResult;
        if (paragraph == null)
        {
            Console.WriteLine("Erreur, la requête n'a rien donné.");
            return;
        }
        Console.WriteLine(paragraph.Value as ParagraphDTO);

    }

    void SearchLessonByTitle()
    {
        Console.WriteLine("\nVeuillez saisir le titre de la leçon recherchée : ");
        var title = Console.ReadLine();
        var lesson = lessonController.GetLessonByTitle(title) as OkObjectResult;
        if (lesson == null)
        {
            Console.WriteLine("Erreur, la requête n'a rien donné.");
            return;
        }
        Console.WriteLine(lesson.Value as LessonDTO);

    }

    void SearchSuccessByUserId()
    {
        Console.WriteLine("\nVeuillez saisir l'identifiant de l'utilisateur du succès recherché : ");
        var id = Console.ReadLine();
        var success = successController.GetSuccessByUserId(int.Parse(id)) as OkObjectResult;
        if (success == null)
        {
            Console.WriteLine("Erreur, la requête n'a rien donné.");
            return;
        }
        Console.WriteLine(success.Value as SuccessDTO);

    }

    void SearchUserById()
    {
        Console.WriteLine("\nVeuillez saisir l'identifiant de l'utilisateur recherché : ");
        var id = Console.ReadLine();
        var user = userController.GetUserById(int.Parse(id)) as OkObjectResult;
        if (user == null)
        {
            Console.WriteLine("Erreur, la requête n'a rien donné.");
            return;
        }
        Console.WriteLine(user.Value as UserDTO);

    }

    void SearchInquiryById()
    {
        Console.WriteLine("\nVeuillez saisir l'identifiant de l'enquête recherchée : ");
        var id = Console.ReadLine();
        var inquiry = inquiryController.GetInquiryById(int.Parse(id)) as OkObjectResult;
        if (inquiry == null)
        {
            Console.WriteLine("Erreur, la requête n'a rien donné.");
            return;
        }
        Console.WriteLine(inquiry.Value as InquiryDTO);
    }

    void SearchParagraphById()
    {
        Console.WriteLine("\nVeuillez saisir l'identifiant du paragraphe recherché : ");
        var id = Console.ReadLine();
        var paragraph = paragraphController.GetParagraphById(int.Parse(id)) as OkObjectResult;
        if (paragraph == null)
        {
            Console.WriteLine("Erreur, la requête n'a rien donné.");
            return;
        }
        Console.WriteLine(paragraph.Value as ParagraphDTO);

    }

    void SearchSuccessByInquiryId()
    {
        Console.WriteLine("\nVeuillez saisir l'identifiant de l'enquête du succès recherché : ");
        var id = Console.ReadLine();
        var success = successController.GetSuccessByInquiryId(int.Parse(id)) as OkObjectResult;
        if (success == null)
        {
            Console.WriteLine("Erreur, la requête n'a rien donné.");
            return;
        }
        Console.WriteLine(success.Value as SuccessDTO);

    }

    void SearchLessonById()
    {
        Console.WriteLine("\nVeuillez saisir l'identifiant de la leçon recherchée : ");
        var id = Console.ReadLine();
        var lesson = lessonController.GetLessonById(int.Parse(id)) as OkObjectResult;
        if (lesson == null)
        {
            Console.WriteLine("Erreur, la requête n'a rien donné.");
            return;
        }
        Console.WriteLine(lesson.Value as LessonDTO);

    }

    void AddUser()
    {
        Console.WriteLine("Veuillez saisir le pseudonyme :");
        var username = Console.ReadLine();
        Console.WriteLine("Veuillez saisir une adresse email :");
        var email = Console.ReadLine();
        Console.WriteLine("Veuillez saisir un mot de passe :");
        var mdp = Console.ReadLine();
        var res = userController.CreateUser(new UserDTO(username, mdp, email, false));
        if (res.GetType() == typeof(CreatedResult))
        {
            Console.WriteLine("\nUtilisateur créé avec succès");
        }
        else
        {
            Console.WriteLine("\nErreur lors de la création de l'utilisateur !");
        }

    }

    void AddInquiry()
    {
        Console.WriteLine("Veuillez saisir le titre :");
        var title = Console.ReadLine();
        Console.WriteLine("Veuillez saisir une description :");
        var description = Console.ReadLine();
        Console.WriteLine("Veuillez saisir un isUser (false / true) :");
        var isUser = Convert.ToBoolean(Console.ReadLine());
        Console.WriteLine("Veuillez saisir l'id de la database :");
        var database = Convert.ToInt32(Console.ReadLine());
        Console.WriteLine("Veuillez saisir l'id de la solution :");
        var solution = Convert.ToInt32(Console.ReadLine());
        var res = inquiryController.CreateInquiry(new InquiryDTO(title, description, false, database, solution));
        if (res.GetType() == typeof(CreatedResult))
        {
            Console.WriteLine("\nEnquête créée avec succès");
        }
        else
        {
            Console.WriteLine("\nErreur lors de la création de l'enquête !");
        }
    }

    void AddParagraph()
    {
        Console.WriteLine("Veuillez saisir le titre :");
        var title = Console.ReadLine();
        Console.WriteLine("Veuillez saisir un contenu :");
        var content = Console.ReadLine();
        Console.WriteLine("Veuillez saisir une information :");
        var info = Console.ReadLine();
        Console.WriteLine("Veuillez saisir une query :");
        var query = Console.ReadLine();
        Console.WriteLine("Veuillez saisir un commentaire :");
        var comment = Console.ReadLine();
        Console.WriteLine("Veuillez saisir l'id de la leçon :");
        var lesson = Convert.ToInt32(Console.ReadLine());
        var res = paragraphController.CreateParagraph(new ParagraphDTO(title, content, info, query, comment, lesson));
        if (res.GetType() == typeof(CreatedResult))
        {
            Console.WriteLine("\nParagraphe créé avec succès");
        }
        else
        {
            Console.WriteLine("\nErreur lors de la création du paragraphe !");
        }

    }

    void AddLesson()
    {
        Console.WriteLine("Veuillez saisir le titre :");
        var title = Console.ReadLine();
        Console.WriteLine("Veuillez saisir votre nom :");
        var lastPublisher = Console.ReadLine();
        var res = lessonController.CreateLesson(new LessonDTO(title, lastPublisher, DateOnly.FromDateTime(DateTime.Now)));
        if (res.GetType() == typeof(CreatedResult))
        {
            Console.WriteLine("\nLeçon créée avec succès");
        }
        else
        {
            Console.WriteLine("\nErreur lors de la création de la leçon !");
        }

    }

    void AddSuccess()
    {
        Console.WriteLine("Veuillez saisir l'identifiant de l'utilisateur lié au succès :");
        var userId = Console.ReadLine();
        Console.WriteLine("Veuillez saisir l'identifiant de l'enquête lié au succès :");
        var inquiryId = Console.ReadLine();
        Console.WriteLine("Veuillez indiquer si l'enquête a été complété (true/false) :");
        var isFinished = Console.ReadLine();
        var res = successController.CreateSuccess(new SuccessDTO(int.Parse(userId), int.Parse(inquiryId), bool.Parse(isFinished)));
        if (res.GetType() == typeof(CreatedResult))
        {
            Console.WriteLine("\nSuccès créé avec succès");
        }
        else
        {
            Console.WriteLine("\nErreur lors de la création du succès !");
        }

    }

    void UpdateUser()
    {
        Console.WriteLine("Quel est l'identifiant de l'utilisateur à mettre à jour ?");
        var id = int.Parse(Console.ReadLine());
        var res = (userController.GetUserById(id));
        if (res.GetType() == typeof(OkObjectResult))
        {
            var user = (res as OkObjectResult).Value as UserDTO;
            if (user == null)
            {
                Console.WriteLine("Erreur, un problème est survenu");
                return;
            }
            else
            {
                Console.WriteLine("Utilisateur trouvé !\n");
                Console.WriteLine("Veuillez saisir le pseudonyme :");
                var username = Console.ReadLine();
                Console.WriteLine("Veuillez saisir l'email :");
                var email = Console.ReadLine();
                var retour = userController.UpdateUser(id, new UserDTO(id, username, user.Password, email, user.IsAdmin));
                if (retour.GetType() == typeof(OkObjectResult))
                {
                    Console.WriteLine("Mise à jour effectué avec succès !");
                }
                else
                {
                    Console.WriteLine("Une erreur est survenue lors de la mise à jour.");
                }
            }
        }
        else
        {
            Console.WriteLine("Une erreur est survenue lors de la mise à jour !");
        }

    }

    void UpdateInquiry()
    {
        Console.WriteLine("Quel est l'identifiant de l'enquête à mettre à jour ?");
        var id = int.Parse(Console.ReadLine());
        var res = (inquiryController.GetInquiryById(id));
        if (res.GetType() == typeof(OkObjectResult))
        {
            var user = (res as OkObjectResult).Value as InquiryDTO;
            if (user == null)
            {
                Console.WriteLine("Erreur, un problème est survenu");
                return;
            }
            else
            {
                Console.WriteLine("Enquête trouvée !\n");
                Console.WriteLine("Veuillez saisir le titre :");
                var title = Console.ReadLine();
                Console.WriteLine("Veuillez saisir la description :");
                var description = Console.ReadLine();
                Console.WriteLine("Veuillez indiquer si l'enquête est accessible aux visiteurs (0/1) :");
                var isUser = Console.ReadLine();
                Console.WriteLine("Veuillez saisir un commentaire :");
                var database = Convert.ToInt32(Console.ReadLine());
                Console.WriteLine("Veuillez saisir un commentaire :");
                var inquiryTable = Convert.ToInt32(Console.ReadLine());
                var retour = inquiryController.UpdateInquiry(id, new InquiryDTO(id, title, description, bool.Parse(isUser), database, inquiryTable));
                if (retour.GetType() == typeof(OkObjectResult))
                {
                    Console.WriteLine("Mise à jour effectué avec succès !");
                }
                else
                {
                    Console.WriteLine("Une erreur est survenue lors de la mise à jour.");
                }
            }
        }
        else
        {
            Console.WriteLine("Une erreur est survenue lors de la mise à jour !");
        }

    }

    void UpdateParagraph()
    {
        Console.WriteLine("Quel est l'identifiant du paragraphe à mettre à jour ?");
        var id = int.Parse(Console.ReadLine());
        var res = (paragraphController.GetParagraphById(id));
        if (res.GetType() == typeof(OkObjectResult))
        {
            var paragraph = (res as OkObjectResult).Value as ParagraphDTO;
            if (paragraph == null)
            {
                Console.WriteLine("Erreur, un problème est survenu");
                return;
            }
            else
            {
                Console.WriteLine("Paragraphe trouvé !\n");
                Console.WriteLine("Veuillez saisir le titre :");
                var title = Console.ReadLine();
                Console.WriteLine("Veuillez saisir un contenu :");
                var content = Console.ReadLine();
                Console.WriteLine("Veuillez saisir une information :");
                var info = Console.ReadLine();
                Console.WriteLine("Veuillez saisir une query :");
                var query = Console.ReadLine();
                Console.WriteLine("Veuillez saisir un commentaire :");
                var comment = Console.ReadLine();
                Console.WriteLine("Veuillez saisir l'id de la leçon :");
                var lesson = Convert.ToInt32(Console.ReadLine());
                var retour = paragraphController.UpdateParagraph(id, new ParagraphDTO(id, title, content, info, query, comment, lesson));
                if (retour.GetType() == typeof(OkObjectResult))
                {
                    Console.WriteLine("Mise à jour effectué avec succès !");
                }
                else
                {
                    Console.WriteLine("Une erreur est survenue lors de la mise à jour.");
                }
            }
        }
        else
        {
            Console.WriteLine("Une erreur est survenue lors de la mise à jour !");
        }

    }

    void UpdateLesson()
    {
        Console.WriteLine("Quel est l'identifiant de la leçon à mettre à jour ?");
        var id = int.Parse(Console.ReadLine());
        var res = (lessonController.GetLessonById(id));
        if (res.GetType() == typeof(OkObjectResult))
        {
            var lesson = (res as OkObjectResult).Value as LessonDTO;
            if (lesson == null)
            {
                Console.WriteLine("Erreur, un problème est survenu");
                return;
            }
            else
            {
                Console.WriteLine("Leçon trouvée !\n");
                Console.WriteLine("Veuillez saisir le titre :");
                var title = Console.ReadLine();
                Console.WriteLine("Veuillez saisir votre nom :");
                var lastPublisher = Console.ReadLine();
                var retour = lessonController.UpdateLesson(id, new LessonDTO(id, title, lastPublisher, DateOnly.FromDateTime(DateTime.Now)));
                if (retour.GetType() == typeof(OkObjectResult))
                {
                    Console.WriteLine("Mise à jour effectué avec succès !");
                }
                else
                {
                    Console.WriteLine("Une erreur est survenue lors de la mise à jour.");
                }
            }
        }
        else
        {
            Console.WriteLine("Une erreur est survenue lors de la mise à jour !");
        }

    }

    void UpdateSuccess()
    {
        Console.WriteLine("Quel est l'identifiant de l'utilisateur lié au succès à mettre à jour ?");
        var id = int.Parse(Console.ReadLine());
        var res = (successController.GetSuccessByUserId(id));
        if (res.GetType() == typeof(OkObjectResult))
        {
            var lesson = (res as OkObjectResult).Value as SuccessDTO;
            if (lesson == null)
            {
                Console.WriteLine("Erreur, un problème est survenu");
                return;
            }
            else
            {
                Console.WriteLine("Succès trouvé !\n");
                Console.WriteLine("Veuillez saisir l'identifiant de l'utilisateur lié au succès :");
                var userId = Console.ReadLine();
                Console.WriteLine("Veuillez saisir l'identifiant de l'enquête lié au succès :");
                var inquiryId = Console.ReadLine();
                Console.WriteLine("Veuillez saisir si l'enquête est terminée (0/1) :");
                var isFinished = Console.ReadLine();
                var retour = successController.UpdateSuccess(id, new SuccessDTO(int.Parse(userId), int.Parse(inquiryId), bool.Parse(isFinished)));
                if (retour.GetType() == typeof(OkObjectResult))
                {
                    Console.WriteLine("Mise à jour effectué avec succès !");
                }
                else
                {
                    Console.WriteLine("Une erreur est survenue lors de la mise à jour.");
                }
            }
        }
        else
        {
            Console.WriteLine("Une erreur est survenue lors de la mise à jour !");
        }

    }

    void DeleteUser()
    {
        Console.WriteLine("Quel est l'identifiant de lutilisateur à supprimer ?");
        var id = int.Parse(Console.ReadLine());
        var res = userController.DeleteUser(id);
        if (res.GetType() == typeof(OkObjectResult))
        {
            Console.WriteLine("La suppression a été effectuée avec succès !");
        }
        else
        {
            Console.WriteLine("Erreur lors de la suppression !");
        }

    }

    void DeleteInquiry()
    {
        Console.WriteLine("Quel est l'identifiant de l'enquête à supprimer ?");
        var id = int.Parse(Console.ReadLine());
        var res = inquiryController.DeleteInquiry(id);
        if (res.GetType() == typeof(OkObjectResult))
        {
            Console.WriteLine("La suppression a été effectuée avec succès !");
        }
        else
        {
            Console.WriteLine("Erreur lors de la suppression !");
        }
    }

    void DeleteParagraph()
    {
        Console.WriteLine("Quel est l'identifiant du paragraphe à supprimer ?");
        var id = int.Parse(Console.ReadLine());
        var res = paragraphController.DeleteParagraph(id);
        if (res.GetType() == typeof(OkObjectResult))
        {
            Console.WriteLine("La suppression a été effectuée avec succès !");
        }
        else
        {
            Console.WriteLine("Erreur lors de la suppression !");
        }

    }

    void DeleteLesson()
    {
        Console.WriteLine("Quel est l'identifiant de la leçon à supprimer ?");
        var id = int.Parse(Console.ReadLine());
        var res = lessonController.DeleteLesson(id);
        if (res.GetType() == typeof(OkObjectResult))
        {
            Console.WriteLine("La suppression a été effectuée avec succès !");
        }
        else
        {
            Console.WriteLine("Erreur lors de la suppression !");
        }

    }

    void DeleteSuccess()
    {
        Console.WriteLine("Quel est l'identifiant de l'utilisateur lié au succès à supprimer ?");
        var id = int.Parse(Console.ReadLine());
        var res = successController.DeleteSuccess(id);
        if (res.GetType() == typeof(OkObjectResult))
        {
            Console.WriteLine("La suppression a été effectuée avec succès !");
        }
        else
        {
            Console.WriteLine("Erreur lors de la suppression !");
        }

    }

    void MenuUsers()
    {
        Console.WriteLine("|------------------------------------------------|");
        Console.WriteLine("|                  MenuUsers                     |");
        Console.WriteLine("|------------------------------------------------|");
        Console.WriteLine("| t - Effectuer des tests automatiques           |");
        Console.WriteLine("| 1 - Afficher les utilisateurs                  |");
        Console.WriteLine("| 2 - Rechercher un utilisateur avec son pseudo  |");
        Console.WriteLine("| 3 - Rechercher un utilisateur avec son ID      |");
        Console.WriteLine("| 4 - Ajouter un utilisateur                     |");
        Console.WriteLine("| 5 - Mettre à jour un utilisateur               |");
        Console.WriteLine("| 6 - Supprimer un utilisateur                   |");
        Console.WriteLine("| q - Quitter                                    |");
        Console.WriteLine("|------------------------------------------------|");
    }

    void MenuInquiries()
    {
        Console.WriteLine("|------------------------------------------------|");
        Console.WriteLine("|                  MenuInquiries                 |");
        Console.WriteLine("|------------------------------------------------|");
        Console.WriteLine("| 1 - Afficher les enquêtes                      |");
        Console.WriteLine("| 2 - Rechercher une enquête avec son titre      |");
        Console.WriteLine("| 3 - Rechercher une enquête avec son ID         |");
        Console.WriteLine("| 4 - Ajouter une enquête                        |");
        Console.WriteLine("| 5 - Mettre à jour une enquête                  |");
        Console.WriteLine("| 6 - Supprimer une enquête                      |");
        Console.WriteLine("| q - Quitter                                    |");
        Console.WriteLine("|------------------------------------------------|");
    }

    void MenuParagraphs()
    {
        Console.WriteLine("|------------------------------------------------|");
        Console.WriteLine("|                  MenuParagraphs                |");
        Console.WriteLine("|------------------------------------------------|");
        Console.WriteLine("| 1 - Afficher les paragraphes                   |");
        Console.WriteLine("| 2 - Rechercher un paragraphe avec son titre    |");
        Console.WriteLine("| 3 - Rechercher un paragraphe avec son ID       |");
        Console.WriteLine("| 4 - Ajouter un paragraphe                      |");
        Console.WriteLine("| 5 - Mettre à jour un paragraphe                |");
        Console.WriteLine("| 6 - Supprimer un paragraphe                    |");
        Console.WriteLine("| q - Quitter                                    |");
        Console.WriteLine("|------------------------------------------------|");
    }

    void MenuLessons()
    {
        Console.WriteLine("|------------------------------------------------|");
        Console.WriteLine("|                  MenuLessons                   |");
        Console.WriteLine("|------------------------------------------------|");
        Console.WriteLine("| 1 - Afficher les leçons                        |");
        Console.WriteLine("| 2 - Rechercher une leçon avec son titre        |");
        Console.WriteLine("| 3 - Rechercher une leçon avec son ID           |");
        Console.WriteLine("| 4 - Ajouter une leçon                          |");
        Console.WriteLine("| 5 - Mettre à jour une leçon                    |");
        Console.WriteLine("| 6 - Supprimer une leçon                        |");
        Console.WriteLine("| q - Quitter                                    |");
        Console.WriteLine("|------------------------------------------------|");
    }

    void MenuSuccesses()
    {
        Console.WriteLine("|------------------------------------------------|");
        Console.WriteLine("|                  MenuSuccesses                 |");
        Console.WriteLine("|------------------------------------------------|");
        Console.WriteLine("| 1 - Afficher les succès                        |");
        Console.WriteLine("| 2 - Rechercher un succès avec son userId       |");
        Console.WriteLine("| 3 - Rechercher un succès avec son inquiryId    |");
        Console.WriteLine("| 4 - Ajouter un succès                          |");
        Console.WriteLine("| 5 - Mettre à jour un succès                    |");
        Console.WriteLine("| 6 - Supprimer un succès                        |");
        Console.WriteLine("| q - Quitter                                    |");
        Console.WriteLine("|------------------------------------------------|");
    }

    MenuUsers();
    //MenuInquiries();
    //MenuParagraphs();
    //MenuLessons();
    //MenuSuccesses();
    Console.WriteLine("\nSaisie :");
    var saisie = Console.ReadLine();

    while (saisie != "q")
    {
        switch (saisie)
        {
            case "1":
                PrintUsers();
                //PrintInquiries();
                //PrintParagraphs();
                //PrintLessons();
                //PrintSuccesses();
                break;
            case "2":
                SearchUserByUsername();
                //SearchInquiryByTitle();
                //SearchParagraphByTitle();
                //SearchLessonByTitle();
                //SearchSuccessByUserId();
                break;
            case "3":
                SearchUserById();
                //SearchInquiryById();
                //SearchParagraphById();
                //SearchLessonById();
                //SearchSuccessByInquiryId();
                break;
            case "4":
                AddUser();
                //AddInquiry();
                //AddParagraph();
                //AddLesson();
                //AddSuccess();
                break;
            case "5":
                UpdateUser();
                //UpdateInquiry();
                //UpdateParagraph();
                //UpdateLesson();
                //UpdateSuccess();
                break;
            case "6":
                DeleteUser();
                //DeleteInquiry();
                //DeleteParagraph();
                //DeleteLesson();
                //UpdateSuccess();
                break;
            case "t":
                AutoTestsUsers();
                break;
            default:
                break;
        }
        Console.WriteLine("\nAppuyez sur n'importe quelle touche pour continuer...");
        Console.ReadKey();
        Console.Clear();
        MenuUsers();
        //MenuInquiries();
        //MenuParagraphs();
        //MenuLessons();
        //MenuSuccesses();
        Console.WriteLine("\nSaisie :");
        saisie = Console.ReadLine();
    }

    void AutoTestsUsers()
    {
        // Affichage des utilisateurs
        Console.WriteLine("\n##########################################################\n");
        Console.WriteLine("Affichages des utilisateurs stubbés dans le contexte :\n");
        var res = userController.GetUsers(1, 10, UserOrderCriteria.None);
        if (res == null)
        {
            Console.WriteLine("\nErreur lors de l'acquisition de la liste des utilisateurs");
        }
        else
        {
            var users = res as IEnumerable<UserDTO>;
            if (users == null)
            {
                Console.WriteLine("\nErreur, les ustilisateurs n'ont pas été trouvés !");
            }
            else
            {
                PrintUsers();
            }
        }

        // Recherche d'utilisateur par ID
        Console.WriteLine("\n##########################################################\n");
        Console.WriteLine("Affichage de l'utilisateur ayant pour identifiant 1 :\n");
        var res1 = userController.GetUserById(1);
        if (res1 == null)
        {
            Console.WriteLine("\nErreur lors de l'acquisition de l'utilisateur !");
        }
        else
        {
            var user = res1;
            if (user == null)
            {
                Console.WriteLine("\nErreur, l'utilisateur n'existe pas !");
            }
            else
            {
                Console.WriteLine(user);
            }
        }

        // Recherche d'utilisateur par pseudonyme
        Console.WriteLine("\n##########################################################\n");
        Console.WriteLine("Affichage de l'utilisateur ayant pour username johnny :\n");
        var res2 = userController.GetUserByUsername("johnny");
        if (res2 == null)
        {
            Console.WriteLine("\nErreur lors de l'acquisition de l'utilisateur !");
        }
        else
        {
            var user1 = res2;
            if (user1 == null)
            {
                Console.WriteLine("\nErreur, l'utilisateur n'existe pas !");
            }
            else
            {
                Console.WriteLine(user1);
            }
        }

        // Ajout d'un utilisateur
        Console.WriteLine("\n##########################################################\n");
        Console.WriteLine("Création de l'utilisateur :\n");
        var user2 = new UserDTO("JohnDoe", "motdepasse", "johndoe@gmail.com", false);
        Console.WriteLine(user2);
        var res3 = userController.CreateUser(user2);
        if (res3 == null)
        {
            Console.WriteLine("\nErreur lors de la création de l'utilisateur !");
        }
        else
        {
            Console.WriteLine("\nUtilisateur créé avec succès !");
        }

        // Affichage des utilisateurs
        Console.WriteLine("\n##########################################################\n");
        PrintUsers();

        // Mise à jour d'un utilisateur
        Console.WriteLine("\n##########################################################\n");
        Console.WriteLine("Mise à jour de l'adresse email de l'utilisateur :\n");
        if (userController.GetUserByUsername("JohnDoe") == null)
        {
            Console.WriteLine("\nErreur lors de la récupération de l'utilisateur !");
        }
        else
        {
            Console.WriteLine(user2);
            Console.WriteLine("\nNouvelle adresse : John.DOE@etu.uca.fr");
            var user3 = new UserDTO(user2.Id, user2.Username, user2.Password, "John.DOE@etu.uca.fr", user2.IsAdmin);
            var res4 = userController.UpdateUser(1, user3);
            if (res4 == null)
            {
                Console.WriteLine("\nErreur lors de la mise à jour de l'utilisateur !");
            }
            else
            {
                Console.WriteLine("\nUtilisateur mis à jour avec succès !");
            }
        }
        // Affichage des utilisateurs

        Console.WriteLine("\n##########################################################\n");
        PrintUsers();

        // Suppression d'un utilisateur
        Console.WriteLine("\n##########################################################\n");
        Console.WriteLine("Suppression de l'utilisateur JohnDoe:\n");
        if (userController.GetUserByUsername("JohnDoe") == null)
        {
            Console.WriteLine("\nErreur lors de la récupération de l'utilisateur !");
        }
        else
        {
            Console.WriteLine(user2);
            var res5 = userController.DeleteUser(user2.Id);
            if (res5 == null)
            {
                Console.WriteLine("\nErreur lors de la suppression de l'utilisateur !");
            }
            else
            {
                Console.WriteLine("\nUtilisateur supprimé avec succès !");
            }
        }
        // Affichage des utilisateurs

        Console.WriteLine("\n##########################################################\n");
        PrintUsers();
    }
}